﻿#region Copyright and License Notice
// Copyright (C)2010-2013 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>
#endregion

using System.Collections.Generic;
using ConfigGen.ConsoleApp.Core.CommandLineOptionCore;
using ConfigGen.Core;

namespace ConfigGen.ConsoleApp.Core.CommandLineOptions
{
    /// <summary>
    /// Console argument handler to handle the "dump settings only" command line option. This processor is responsible for setting the
    /// <see cref="Preferences.DumpSettingsOnly"/> property on the settings object.
    /// </summary>
    public class DumpSettingsOnly : ICommandLineOption<ConsoleAppPreferences>
    {
        #region ICommandLineOption<ConsoleAppPreferences> Members

        /// <summary>
        /// Gets the short name of the command line argument, without the leading hyphen: "dso"
        /// </summary>
        /// <value></value>
        public string ShortName
        {
            get { return "dso"; }
        }

        /// <summary>
        /// Gets the long name of the command line argument, without the leading hyphens: "dump-settings-only"
        /// </summary>
        /// <value></value>
        public string LongName
        {
            get { return "dump-settings-only"; }
        }

        /// <summary>
        /// Gets help text for this command line option.
        /// </summary>
        /// <value></value>
        public string HelpText
        {
            get
            {
                var dumpSettingsFile = new DumpSettingsFile();
                return
                    string.Format(
                        "Causes the program to write it's settings to a file, specified by -{0} or -{1} (one of which is compulsory if this setting is used), and exit.",
                        dumpSettingsFile.ShortName,
                        dumpSettingsFile.LongName);
            }
        }

        /// <summary>
        /// Gets a help string containing the args usage for this option: null for this type
        /// </summary>
        /// <value></value>
        public string ArgsUsage
        {
            get { return null; }
        }

        /// <summary>
        /// Applies the option represented by this instance to the supplied settings instance.
        /// </summary>
        /// <param name="settings">The settings object.</param>
        public void ApplyOptionToSettings(ConsoleAppPreferences settings)
        {
            settings.DumpSettingsOnly = true;
        }

        /// <summary>
        /// Processes the argument queue by consuming (dequeuing) any arguments used by this instance.
        /// </summary>
        /// <param name="args">Args queue from which to consume args</param>
        public void ConsumeArgumentsFromQueue(Queue<string> args)
        {
            
        }

        #endregion
    }
}
