﻿#region Copyright and License Notice
// Copyright (C)2010-2013 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>
#endregion

using System.Collections.Generic;
using ConfigGen.ConsoleApp.Core.CommandLineOptionCore;
using ConfigGen.Core;

namespace ConfigGen.ConsoleApp.Core.CommandLineOptions
{
    /// <summary>
    /// Console argument handler to handle the "machine suffix" command line option. This processor is responsible for setting the
    /// <see cref="Preferences.MachineSuffix"/> property on the settings object.
    /// </summary>
    public class MachineSuffix : ICommandLineOption<ConsoleAppPreferences>
    {
        #region ICommandLineOption<Preferences> Members

        /// <summary>
        /// Gets the short name of the command line argument, without the leading hyphen; for this processor: "m".
        /// </summary>
        /// <value>"m"</value>
        public string ShortName
        {
            get { return "m"; }
        }

        /// <summary>
        /// Gets the long name of the command line argument, without the leading hyphens; for this processor: "machine-suffix"
        /// </summary>
        /// <value>"machine-suffix"</value>
        public string LongName
        {
            get { return "machine-suffix"; }
        }

        /// <summary>
        /// Gets a help string containing the args usage for this processor; null in the case of this type
        /// </summary>
        /// <value>null</value>
        public string ArgsUsage
        {
            get { return null; }
        }

        /// <summary>
        /// Gets help text for this command line option.
        /// </summary>
        /// <value>Help text</value>
        public string HelpText
        {
            get
            {
                return
                    "specifies that each generated config file should be suffixed with the machine name to which it pertains. For example, an App.Config for machine DevServer1 would be output as App.Confing.DevServer1";
            }
        }

        /// <summary>
        /// Applies the option represented by this instance to the supplied settings instance.
        /// </summary>
        /// <param name="settings">The settings object.</param>
        public void ApplyOptionToSettings(ConsoleAppPreferences settings)
        {
            settings.MachineSuffix = true;
        }

        /// <summary>
        /// Processes the argument queue by consuming (dequeuing) any arguments used by this instance. For this type of processor, nothing is consumed
        /// by this method.
        /// </summary>
        /// <param name="args">Args queue from which to consume args</param>
        public void ConsumeArgumentsFromQueue(Queue<string> args)
        {
            // no args to process
        }

        #endregion
    }
}
