﻿#region Copyright and License Notice
// Copyright (C)2010-2012 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>.
#endregion
using System.IO;
using System.Linq;
using System.Reflection;
using System.Xml.Linq;
using ConfigGen.Utilities;
using ConfigGen.Utilities.Extensions.System.Reflection;
using NUnit.Framework;

namespace ConfigGen.Core.Tests.EndToEndTests
{
    [TestFixture]
    public class CsvEndToEndTests
    {
        [Test]
        public void CsvFileCanBeUsed()
        {
            using (var testDirectory = new DisposableDirectory())
            {
                string settingsFile = Path.Combine(testDirectory.FullName, "App.Config.Settings.csv");
                Assembly.GetExecutingAssembly().CopyEmbeddedResourceFileTo("TestResources.App.Config.Settings.csv", settingsFile);

                string templateFile = Path.Combine(testDirectory.FullName, "App.Config.Template.xml");
                Assembly.GetExecutingAssembly().CopyEmbeddedResourceFileTo("TestResources.App.Config.Template.xml", templateFile);

                var settings = new Preferences
                {
                    SettingsFile = settingsFile,
                    TemplateFile = templateFile,
                    OutputDirectory = Path.Combine(testDirectory.FullName, "Configs"),
                };

                var results = TestHelper.ExecuteCfgTest(settings);

                TestHelper.CheckExitResults(results, false);

                var fileDevServer = XElement.Load(Path.Combine(testDirectory.FullName, "Configs\\DevServer\\App.Config"));
              
            }
        }

        [Test]
        public void CsvFile_With_blank_lines_reads_to_end()
        {
            using (var testDirectory = new DisposableDirectory())
            {
                string settingsFile = Path.Combine(testDirectory.FullName, "BlankLines.Config.Settings.csv");
                Assembly.GetExecutingAssembly().CopyEmbeddedResourceFileTo("TestResources.App.Config.Settings.csv", settingsFile);

                string templateFile = Path.Combine(testDirectory.FullName, "App.Config.Template.xml");
                Assembly.GetExecutingAssembly().CopyEmbeddedResourceFileTo("TestResources.App.Config.Template.xml", templateFile);

                var settings = new Preferences
                {
                    SettingsFile = settingsFile,
                    TemplateFile = templateFile,
                    OutputDirectory = Path.Combine(testDirectory.FullName, "Configs"),
                };

                var results = TestHelper.ExecuteCfgTest(settings);

                TestHelper.CheckExitResults(results, false);

                Assert.AreEqual(5, results.Count());

                var fileDevServer = XElement.Load(Path.Combine(testDirectory.FullName, "Configs\\DevServer\\App.Config"));

            }
        }

    }
}
