﻿#region Copyright and License Notice
// Copyright (C)2010-2013 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>
#endregion

using System.IO;

namespace ConfigGen.Core.TemplateLoader
{
    /// <summary>
    /// Interface implemented by classes that load config gen templates.
    /// </summary>
    public interface ITemplateLoader
    {
        /// <summary>
        /// Loads the template according to the supplied arguments (e.g. a filename for a file loader)
        /// </summary>
        /// <param name="args">Loader arguments</param>
        void LoadTemplate(string[] args);

        /// <summary>
        /// Returns a stream containing the raw temaple bytes.
        /// </summary>
        /// <returns>A stream containing the raw temaple bytes.</returns>
        Stream GetTemplateStream();

        /// <summary>
        /// Gets the template preferences loaded by the instance, if any, otherwise null.
        /// </summary>
        TemplatePreferences TemplatePreferences { get; }
    }
}