﻿#region Copyright and License Notice
// Copyright (C)2010-2013 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>
#endregion

using System;
using ConfigGen.Core.ConfigurationSettings;

namespace ConfigGen.Core.TokenHandling
{
    /// <summary>
    /// Interface implemented by <see cref="TokenReplacer"/>
    /// </summary>
    public interface ITokenReplacer
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="TokenReplacer"/> class.
        /// </summary>
        /// <param name="configConfigurationSettingsForMachineConfiguration">The config settings for machine.</param>
        /// <exception cref="ArgumentNullException">Thrown if <paramref name="configConfigurationSettingsForMachineConfiguration"/> is null.</exception>
        /// <exception cref="InvalidOperationException">Thrown if this method is called more than once.</exception>
        void Initialise(MachineConfigurationSettings configConfigurationSettingsForMachineConfiguration);

        /// <summary>
        /// Returns the value of the supplied token, if present in the machine settings collection, otherwise throws an exception.
        /// </summary>
        /// <param name="token">Token for which to supply a value</param>
        /// <returns>Token value</returns>
        /// <exception cref="InvalidOperationException">Thrown if this method is called before <see cref="TokenReplacer.Initialise"/> has been called.</exception>
        /// <exception cref="NoMatchingTokenException">Thrown if the supplied token is not present in the machine settings collection.</exception>
        string GetTokenValue(string token);
    }
}